<?php declare(strict_types = 1);

namespace Drupal\sdc_styleguide\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Link;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\Url;
use Drupal\sdc\ComponentPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Finder\Finder;

/**
 * Returns responses for Single Directory Components Styleguide routes.
 */
final class StyleGuideController extends ControllerBase {

  /**
   * The controller constructor.
   */
  public function __construct(
    private readonly ComponentPluginManager $pluginManagerSdc,
    private readonly ThemeHandlerInterface $themeHandler
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('plugin.manager.sdc'),
      $container->get('theme_handler'),
    );
  }

  /**
   * Builds the welcome page.
   */
  public function welcome() {
    return [
      '#theme' => 'styleguide_welcome_message',
    ];
  }

  private function getComponents() {
    $component_tree = [
      'module' => [],
      'theme' => [],
    ];
    foreach ($this->pluginManagerSdc->getAllComponents() as $component) {
      $definition = $component->getPluginDefinition();
      $type = $definition['extension_type']->value;
      $component_full_id = explode(':', $component->getPluginId());
      $extension_id = reset($component_full_id);
      $component_id = end($component_full_id);

      $name = $type == 'theme' ? $this->themeHandler->getName($extension_id) : $this->moduleHandler()->getName($extension_id);
      if (!isset($component_tree[$type][$extension_id])) {
        $component_tree[$type][$extension_id] = [
          'name' => $name,
          'components' => [],
        ];
      }

      // Adds component.
      $component_tree[$type][$extension_id]['components'][$component_id] = [
        'name' => $definition['name'],
        'demos' => [],
      ];

      $demos = &$component_tree[$type][$extension_id]['components'][$component_id]['demos'];

      $finder = new Finder();
      $component_name = $definition['machineName'];
      $finder->in($definition['path'])->files()->name("{$component_name}.demo.*.yml");
      foreach ($finder as $file) {
        $key = str_replace(['.yml', "{$component_name}.demo."], '', $file->getFilename());
        $contents = $file->getContents();
        $demo_data = Yaml::decode($contents);
        $demos[$key] = $demo_data;
      }

      // Sorts by component readable name.
      uasort($demos, fn ($a, $b) => strcmp($a['name'], $b['name']));
    }


    return $component_tree;
  }

  /**
   * Builds the response.
   */
  public function __invoke(): array {
    $components = $this->getComponents();

    $build = [
      '#prefix' => '<div class="sdc-styleguide-explorer">',
      '#suffix' => '</div>',
    ];
    foreach ($components as $type => $type_instances) {
      $build[$type] = [
        '#markup' => $type,
        '#prefix' => '<h2>',
        '#suffix' => '</h2>',
        'items' => [],
      ];

      $items = &$build[$type]['items'];
      foreach ($type_instances as $type_id => $type_components) {
        $items[$type_id] = [
          '#markup' => $type_components['name'],
          '#prefix' => '<h3>',
          '#suffix' => '</h3>',
          'items' => [],
        ];

        $components = &$items[$type_id]['items'];
        foreach ($type_components['components'] as $component_id => $definition) {
          $components[$component_id] = [
            'heading' => [
              '#markup' => $definition['name'],
              '#prefix' => '<h4>',
              '#suffix' => '</h4>',
            ],
            'demos' => [
              '#theme' => 'item_list',
              '#items' => [],
            ],
          ];

          $demos = &$components[$component_id]['demos']['#items'];
          foreach ($definition['demos'] as $demo_id => $data) {
            $demos[] = Link::createFromRoute($data['name'], 'sdc_styleguide.viewer', [
              'view_type' => $type,
              'view_type_instance' => $type_id,
              'view_component' => $component_id,
              'view_demo' => $demo_id,
            ], [
              'attributes' => [
                'class' => ['sdc-styleguide-explorer__link'],
              ],
            ]);
          }
        }
      }
    }

    _sdc_styleguide_page_variables([
      'sidebar' => $build,
      'content' => [
        '#theme' => 'styleguide_component_viewer',
        '#url' => Url::fromRoute('sdc_styleguide.welcome'),
      ],
    ]);
    return ['#markup' => ''];
  }

  public function view(string $view_type, string $view_type_instance, string $view_component, string $view_demo) {
    $components = $this->getComponents();
    $demo = $components[$view_type][$view_type_instance]['components'][$view_component]['demos'][$view_demo];
    return [
      '#type' => 'component',
      '#component' => "{$view_type_instance}:$view_component",
      '#props' => $demo['properties'],
      '#slots' => array_map(fn ($x) => ['#type' => 'inline_template', '#template' => $x], $demo['slots']),
    ];
  }

}
