<?php declare(strict_types = 1);

namespace Drupal\sdc_styleguide\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\sdc\ComponentPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Finder\Finder;

/**
 * Provides a Single Directory Components Styleguide form.
 */
final class SDCExplorerForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'sdc_styleguide_sdc_explorer';
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(protected ComponentPluginManager $componentPluginManager) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.sdc')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $options = [];
    $all_components = [];
    foreach ($this->componentPluginManager->getAllComponents() as $component) {
      $definition = $component->getPluginDefinition();
      $all_components[$definition['id']] = $component;
      $group_name = "{$definition['extension_type']->name}: {$definition['provider']}";
      if (!isset($options[$group_name])) {
        $options[$group_name] = [];
      }
      $options[$group_name][$definition['id']] = $definition['name'];
    }

    $form['component'] = [
      '#ajax' => [
        'callback' => '::onComponentChange',
        'event' => 'change',
        'wrapper' => 'component-wrapper',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Verifying entry...'),
        ],
      ],
      '#empty_option' => $this->t('- Select a component -'),
      '#options' => $options,
      '#title' => $this->t('Component'),
      '#type' => 'select',
    ];

    $form['component_fields'] = [
      '#attributes' => [
        'id' => 'component-wrapper',
      ],
      '#type' => 'container',
    ];

    $demos = [];
    $fapi_map = [
      'string' => 'textfield',
      'number' => 'number',
      'boolean' => 'checkbox',
    ];
    if ($form_state->getValue('component')) {
      $selected_component = $all_components[$form_state->getValue('component')];
      $definition = $selected_component->getPluginDefinition();
      $component_folder = $definition['path'];
      $finder = new Finder();
      $component_name = $definition['machineName'];
      $finder->in($component_folder)->files()->name("{$component_name}.demo.*.yml");
      foreach ($finder as $file) {
        $key = str_replace(['.yml', "{$component_name}.demo."], '', $file->getFilename());
        $contents = $file->getContents();
        $demo_data = Yaml::decode($contents);
        $demos[$key] = $demo_data;
      }

      $form['component_fields']['selected_component'] = [
        '#tree' => TRUE,
        'component' => [
          '#type' => 'value',
          '#value' => $form_state->getValue('component'),
        ],
      ];

      $form['component_fields']['demo'] = [
        '#ajax' => [
          'callback' => '::onComponentSubmit',
          'event' => 'change',
          'wrapper' => 'result',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Verifying entry...'),
          ],
        ],
        '#empty_option' => $this->t('- Select a Demo -'),
        '#options' => array_map(fn ($x) => $x['name'], $demos),
        '#type' => 'select',
      ];

    }

    $form['rendered_result'] = [
      '#attributes' => [
        'id' => 'result',
      ],
      '#type' => 'container',
    ];

    if ($demo = $form_state->getValue('demo')) {
      $component = $form_state->getValue('component');
      $demo = $demos[$demo];
      foreach ($demo['slots'] as &$slot) {
        $slot = [
          '#markup' => $slot,
        ];
      }
      $form['rendered_result']['component'] = [
        '#type' => 'component',
        '#component' => $component,
      ];

      if (!empty($demo['properties'])) {
        $form['rendered_result']['component']['#props'] = $demo['properties'];
      }

      if (!empty($demo['slots'])) {
        $form['rendered_result']['component']['#slots'] = $demo['slots'];
      }
    }

    return $form;
  }

  /**
   * AJAX handler for when the component selector field is changed.
   */
  public function onComponentChange(array &$form, FormStateInterface $form_state) {
    return $form['component_fields'];
  }

  /**
   * AJAX handler for when the component values are submitted.
   */
  public function onComponentSubmit(array &$form, FormStateInterface $form_state) {
    return $form['rendered_result'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->messenger()->addStatus($this->t('The message has been sent.'));
    $form_state->setRedirect('<front>');
  }

}
